#include "commons.h"
#include "mtget.h"
#include"connthread.h"

mtget::mtget(const QString & _url):
nc(10)
{
	url=new QUrl((_url));
	if(!url->isValid())
	return;
	setFileName(url->path());
	connect(this,SIGNAL(ready()),this,SLOT(createThreads()));
	if (url->scheme()=="ftp"){
	ftpInfo=new QFtp(this);
	ftpInfo->connectToHost(url->host());
        ftpInfo->login();
	connect(ftpInfo,SIGNAL(listInfo (const QUrlInfo &)),this,SLOT(ftpFileInfo(const QUrlInfo &)));
        ftpInfo->list(url->path());
	ftpInfo->close();
	}	
	if (url->scheme()=="http"){
	httpInfo=new QHttp(this);
	httpInfo->setHost(url->host());
	connect(httpInfo,SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)),this,SLOT(httpFileInfo(const QHttpResponseHeader &)));
	httpInfo->head(url->path());
	httpInfo->close();
	}
}

void mtget::httpFileInfo(const QHttpResponseHeader & resp)
{
	disconnect(httpInfo,SIGNAL(responseHeaderReceived(const QHttpResponseHeader &)),this,SLOT(httpFileInfo(const QHttpResponseHeader &)));
	setFileAtributes(resp.contentLength());
	return;
}

void mtget::ftpFileInfo(const QUrlInfo & i)
{
	disconnect(ftpInfo,SIGNAL(listInfo (const QUrlInfo &)),this,SLOT(ftpFileInfo(const QUrlInfo &)));
	setFileAtributes(i.size());
	return;
}

void mtget::setFileName(const QString & path)
{
	QFileInfo fileInfo(path);
	fileName = fileInfo.fileName();
	return;
}

void mtget::setFileAtributes(uint length)
{
	filelength=length;
QString infoStr=tr("downloading: %1\tsize:%2 bytes\n").arg(fileName).arg(filelength);
	std::printf(infoStr.toAscii());
QString errorStr;
	if (QFile::exists(fileName)) {
	errorStr=tr("%1 already exists in the current directory.\n").arg(fileName);
	std::printf(errorStr.toAscii());
	return;
        }
	QString filePart=fileName;
	filePart+=".part";
	if (QFile::exists(filePart)) {
/*	errorStr=tr("%1 already exists in the current directory.\n").arg(filePart);
	std::printf(errorStr.toAscii());
	emit done();*/
	file=new QFile();
	file->setFileName(filePart);
	if (file->open(QIODevice::WriteOnly))
	loadjob();
	return;
        }
	file=new QFile();
	file->setFileName(filePart);
	if (!file->open(QIODevice::WriteOnly)) {
	errorStr=tr("Unable to open the file %1: %2.").arg(fileName).arg(file->errorString());
	std::printf(errorStr.toAscii());
	return;
        }
	fileChunks=filelength/BUFFER_SIZE;
	emit ready();
	return;
}

void mtget::connFinished(cfg _conf)
{
	connections.remove(_conf.id);
	std::printf("closing thread-%d\n",_conf.id);
	writeThreadSettings(_conf);
	if(_conf.bytes!=0){
	createThread(_conf.id, file, url, _conf.bytes, _conf.offSet);
	std::printf("restarting thread-%d...\n",_conf.id);		
	}
	if(connections.isEmpty()){
	file->rename(fileName);
	std::printf("done\n");		
	emit done();
	}
	return;

}

void mtget::createThreads()
{
	if(fileChunks<nc)
	nc=fileChunks;
	writeGlobalSettings();
	uint conn_size=filelength/nc;
	uint rest_size=conn_size+(filelength%nc);
	for(int i=0;i<nc;i++){
	if(i==nc-1)
	createThread(i, file, url, rest_size, i*conn_size);
	createThread(i, file, url, conn_size, i*conn_size);
	}
}

void mtget::createThread(int _id, QFile *_file, QUrl *_url, uint _bytes, uint _offSet)
{
	connections[_id]= new connection( _id, _file, _url,  _bytes, _offSet);
	connect(connections.value(_id),SIGNAL(connfinished(cfg)),this,SLOT(connFinished(cfg)));
	connect(connections.value(_id),SIGNAL(savecfg(cfg)),this,SLOT(writeThreadSettings(cfg)));
	connections.value(_id)->start();
	std::printf("starting thread-%d\n",_id);

}

void mtget::readGlobalSettings()
{
	QSettings settings("Nolis71cu", "MultiThreadedGet");
	if(settings.contains(fileName)){
	settings.beginGroup(fileName);
	filelength = settings.value("size").toUInt();
	nc = settings.value("threads").toInt();
	settings.endGroup();
	}
}

cfg mtget::readThreadSettings(int _id)
{
	cfg conf;
	QSettings settings("Nolis71cu", "MultiThreadedGet");
	settings.beginGroup(fileName);
	if(settings.contains(QString::number(_id))){
	settings.beginGroup(QString::number(_id));
	conf.id = _id;
	conf.offSet = settings.value("offSet").toUInt();
	conf.bytes = settings.value("bytes").toUInt();
	settings.endGroup();
	}else
	conf.id=-1;
	settings.endGroup();
	return conf;
}

void mtget::writeGlobalSettings()
{
	settings = new QSettings("Nolis71cu", "MultiThreadedGet");
	settings->beginGroup(fileName);
	settings->setValue("size", filelength);
	settings->setValue("threads", nc);
	settings->endGroup();
}

void mtget::writeThreadSettings(cfg _conf)
{
	std::printf("writing status\n");
	settings = new QSettings("Nolis71cu", "MultiThreadedGet");
	settings->beginGroup(fileName);
	settings->beginGroup(QString::number(_conf.id));
	settings->setValue("offSet", _conf.offSet);
	settings->setValue("bytes", _conf.bytes);
	settings->endGroup();
	settings->endGroup();
}

void mtget::loadjob()
{
	cfg conf;
	readGlobalSettings();
	for(int i=0;i<nc;i++){
	conf = readThreadSettings(i);
	if(!conf.id==-1)
	createThread(conf.id, file, url, conf.bytes, conf.offSet);
	}
	std::printf("restarting job\n");
}
